// Copyright (C) 2014 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of NV Libraries.
// For conditions of distribution and use, see copyright notice in nv.hh

// WARNING: this file is explicitly designed to fuck with your brain

#ifndef NV_INTERPOLATION_RAW_HH
#define NV_INTERPOLATION_RAW_HH

#include <nv/core/common.hh>
#include <nv/core/transform.hh>
#include <nv/core/math.hh>
#include <nv/interface/animation_key.hh>

namespace nv
{

	inline uint32 interpolate_raw_linear( uint32 count, float factor, const float* k1, const float* k2, float* result )
	{
		for ( uint32 i = 0; i < count; ++i )
		{
			result[i] = k1[i] + factor * (k2[i] - k1[i]); 
		}
		return count;
	}

	inline uint32 interpolate_raw_quat( float factor, const float* k1, const float* k2, float* result )
	{
		*((quat*)(result)) = interpolate( *((quat*)(k1)), *((quat*)(k2)), factor );
		return 4;
	}

	inline uint32 interpolate_raw( const key_descriptor_slot& slot, float factor, const float* k1, const float* k2, float* result )
	{
		uint32 count = get_datatype_info( slot.etype ).elements;
		switch ( slot.vslot )
		{
		case animation_slot::TIME:     return 0;
		case animation_slot::POSITION: return interpolate_raw_linear( count, factor, k1, k2, result );
		case animation_slot::ROTATION: return interpolate_raw_quat( factor, k1, k2, result );
		case animation_slot::SCALE:    return interpolate_raw_linear( count, factor, k1, k2, result );
		case animation_slot::TFORM:    return 
										   interpolate_raw_linear( 3, factor, k1, k2, result ) + 
										   interpolate_raw_quat( factor, k1 + 3, k2 + 3, result + 3 );
		default:
			return 0;
		};
	}

	inline quat make_quat_fixed( const float* data )
	{
		quat result;
		memcpy((float*)(&result), data, sizeof(quat));
		return result;
	}

	inline mat4 extract_matrix_raw( const key_descriptor& desc, const float* data )
	{
		if ( desc.count == 1 )
		{
			switch ( desc.slots[0].vslot )
			{
			case animation_slot::TIME:     return mat4();
			case animation_slot::POSITION: return glm::translate( mat4(),glm::make_vec3( data ) );
			case animation_slot::ROTATION: return glm::mat4_cast( make_quat_fixed( data ) );
			case animation_slot::SCALE:    return glm::scale( mat4(),glm::make_vec3( data ) );
			case animation_slot::TFORM:    return transform( glm::make_vec3( data ), make_quat_fixed( data + 3 ) ).extract();
			default:
				return mat4();
			};
		}
		else
		{
			mat4 position;
			mat4 rotation;
			mat4 scale;
			for ( uint32 i = 0; i < desc.count; ++i )
			{
				uint32 offset = desc.slots[i].offset / 4;
				switch ( desc.slots[i].vslot )
				{
				case animation_slot::TIME:     break;
				case animation_slot::POSITION: 
					position = glm::translate( position,glm::make_vec3( data + offset ) ); break;
				case animation_slot::ROTATION: 
					rotation = glm::mat4_cast( make_quat_fixed( data + offset ) ); break;
				case animation_slot::SCALE:    
					scale    = glm::scale( mat4(),glm::make_vec3( data + offset ) ); break;
				case animation_slot::TFORM:    return transform( glm::make_vec3( data + offset ), make_quat_fixed( data + offset + 3 ) ).extract();
				default:
					break;
				}
			}
			return position * rotation * scale;
		}
	}

	inline transform extract_transform_raw( const key_descriptor& desc, const float* data )
	{
		if ( desc.count == 1 )
		{
			switch ( desc.slots[0].vslot )
			{
			case animation_slot::TIME:     return transform();
			case animation_slot::POSITION: return transform( glm::make_vec3( data ) );
			case animation_slot::ROTATION: return transform( make_quat_fixed( data ) );
			case animation_slot::SCALE:    return transform();
			case animation_slot::TFORM:    return transform( glm::make_vec3( data ), make_quat_fixed( data + 3 ) );
			default:
				return transform();
			};
		}
		else
		{
			vec3 position;
			quat rotation;
			for ( uint32 i = 0; i < desc.count; ++i )
			{
				uint32 offset = desc.slots[i].offset / 4;
				switch ( desc.slots[i].vslot )
				{
				case animation_slot::TIME:     break;
				case animation_slot::POSITION: 
					position = glm::make_vec3( data + offset ); break;
				case animation_slot::ROTATION: 
					rotation = make_quat_fixed( data + offset ); break;
				case animation_slot::SCALE:	   break;
				case animation_slot::TFORM:    return transform( glm::make_vec3( data + offset ), make_quat_fixed( data + 3 ) );
				default:
					break;
				}
			}
			return transform( position, rotation );
		}
	}

	inline void transform_key_position( uint8* data, float scale, const mat3& r33 )
	{
		vec3& p = *((vec3*)(data));
		p = r33 * p * scale;
	}
	inline void transform_key_rotation( uint8* data, const mat3& r33, const mat3& ri33 )
	{
		quat& r = *((quat*)(data));
		r = glm::quat_cast( r33 * glm::mat3_cast( r ) * ri33 );
	}
	inline void transform_key_scale( void* data, float scale )
	{
		vec3& s = *((vec3*)(data));
		s = s * scale;
	}
	inline void transform_key_transform( uint8* data, float scale, const mat3& r33, const mat3& ri33 )
	{
		transform& t = *((transform*)(data));
		t = transform( 
			r33 * t.get_position() * scale,
			glm::quat_cast( r33 * glm::mat3_cast( t.get_orientation() ) * ri33 )
		);
	}

	inline void transform_key_raw( const key_descriptor& desc, uint8* data, float scale, const mat3& r33, const mat3& ri33 )
	{
		for ( uint32 i = 0; i < desc.count; ++i )
		{
			uint32 offset = desc.slots[i].offset;
			switch ( desc.slots[i].vslot )
			{
			case animation_slot::TIME:     break;
			case animation_slot::POSITION: transform_key_position( data + offset, scale, r33 ); break;
			case animation_slot::ROTATION: transform_key_rotation( data + offset, r33, ri33 ); break;
			case animation_slot::SCALE:    transform_key_scale( data + offset, scale ); break;
			case animation_slot::TFORM:    transform_key_transform( data + offset, scale, r33, ri33 ); break;
			default:
				break;
			}
		}
	}

}

#endif // NV_INTERPOLATION_RAW_HH
